% This Module computes the Aerodynamic Forces on every panel 
% of the surfaces and its sum etcetera..
%
% Outputs:
% -> Total Forces in wind axis: Lift, Drag, Sideforce
% -> Total Forces in global axis: Fx, Fy, Fz
% -> Total Moments in global axis: Mx, My, Mz, calculated in CG
% -> Coefficient of Pressure
% -> you may modify it to output other details if you wish..

global plott alpha0 beta0 Vinf rho

% Creating the Linear System matrix "MAT"
MAT = Vortex_Lattice_INDUCEDVELOCITYFUNCTION('creating matrix of influence coeffients..');

% Reference Point is the Center of Gravity
refPoint = CG;
if component_aerodynamic_forces == 1
	[Fx,Fy,Fz,M,L,D,S,dFVECT,Cp,FORCE_MOM] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,0,0,0,0,0,0,refPoint);
else
	[Fx,Fy,Fz,M,L,D,S,dFVECT,Cp] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,0,0,0,0,0,0,refPoint);
end

% ___________________________________________
% Writing results in a file
% ___________________________________________

% #####################################################################################################
% #####################################################################################################
% 			Total Aerodynamic Forces
% #####################################################################################################
% #####################################################################################################
if total_aerodynamic_forces == 1
	fid = fopen('Calculated_Total_Aerodynamic_Forces.dat','w');
	
	fprintf(fid,'%% --------------------------------------------------------------\n');
	fprintf(fid,'%% This file contains the Aerodynamic Forces calculated\n');
	fprintf(fid,'%% in the conditions specified in the file "Initial_Conditions.m"\n');
	fprintf(fid,'%% Forces and moments are calculated in BODY AXIS, which are in the same\n');
	fprintf(fid,'%% direction as global axis, BUT x_body is from CG to the nose, y_body is\n');
	fprintf(fid,'%% from CG to the right wing (exactly as y_global), z_body from CG to the \n');
	fprintf(fid,'%% bottom of the airplane.\n');
	fprintf(fid,'%% Moments are calculated in the refPoint position.\n');
	fprintf(fid,'%%\n%% Trasforming body axis into global axis is pretty easy:\n');
	fprintf(fid,'%% Fx_global = -Fx_body; Fy_global = Fy_body; Fz_global = -Fz_body\n%%');
	fprintf(fid,'%% Mx_global = -Mx_body; My_global = My_body; Mz_global = -Mz_body\n%%');
	fprintf(fid,'%% --------------------------------------------------------------\n%%\n');
	fprintf(fid,'%% Alpha [rad]\n%f\n',alpha0);
	fprintf(fid,'%% Beta [rad]\n%f\n',beta0);
	fprintf(fid,'%% Lift [N]\n%f\n',L);
	fprintf(fid,'%% Drag [N]\n%f\n',D);
	fprintf(fid,'%% Side Force [N]\n%f\n',S);
	fprintf(fid,'%% X axis Force [N]\n%f\n',Fx);
	fprintf(fid,'%% Y axis Force [N]\n%f\n',Fy);
	fprintf(fid,'%% Z axis Force [N]\n%f\n',Fz);
	fprintf(fid,'%% X axis Moment [Nm]\n%f\n',M(1));
	fprintf(fid,'%% Y axis Moment [Nm]\n%f\n',M(2));
	fprintf(fid,'%% Z axis Moment [Nm]\n%f\n',M(3));

	fclose(fid);
end % if

% #####################################################################################################
% #####################################################################################################
% 				Component Aerodynamic Forces
% #####################################################################################################
% #####################################################################################################

if component_aerodynamic_forces == 1


	Component_Lifts 	= FORCE_MOM{1};
	Component_Drags 	= FORCE_MOM{2};
	Component_SideForces 	= FORCE_MOM{3};
	Component_Moments	= FORCE_MOM{4};
	Component_Fxs		= FORCE_MOM{5};
	Component_Fys		= FORCE_MOM{6};
	Component_Fzs		= FORCE_MOM{7};

	% Wing
	Lwing 	= Component_Lifts(1);
	Dwing 	= Component_Drags(1);
	Swing 	= Component_SideForces(1);
	Mwing 	= Component_Moments(1,:);
	Fxwing	= Component_Fxs(1);
	Fywing	= Component_Fys(1);
	Fzwing	= Component_Fzs(1);

	% Horizontal Tail
	Lhtail = Component_Lifts(2);
	Dhtail = Component_Drags(2);
	Shtail = Component_SideForces(2);
	Mhtail = Component_Moments(2,:);
	Fxhtail	= Component_Fxs(2);
	Fyhtail	= Component_Fys(2);
	Fzhtail	= Component_Fzs(2);

	% Vertical Tail
	Lvtail = Component_Lifts(3);
	Dvtail = Component_Drags(3);
	Svtail = Component_SideForces(3);
	Mvtail = Component_Moments(3,:);
	Fxvtail	= Component_Fxs(3);
	Fyvtail	= Component_Fys(3);
	Fzvtail	= Component_Fzs(3);

	fid = fopen('Calculated_Component_Aerodynamic_Forces.dat','w');
	
	fprintf(fid,'%% --------------------------------------------------------------\n');
	fprintf(fid,'%% This file contains the Aerodynamic Forces calculated\n');
	fprintf(fid,'%% in the conditions specified in the file "Initial_Conditions.m"\n');
	fprintf(fid,'%% Forces and moments are calculated in BODY AXIS, which are in the same\n');
	fprintf(fid,'%% direction as global axis, BUT x_body is from CG to the nose, y_body is\n');
	fprintf(fid,'%% from CG to the right wing (exactly as y_global), z_body from CG to the \n');
	fprintf(fid,'%% bottom of the airplane.\n');
	fprintf(fid,'%% Moments are calculated in the refPoint position.\n');
	fprintf(fid,'%%\n%% Trasforming body axis into global axis is pretty easy:\n');
	fprintf(fid,'%% Fx_global = -Fx_body; Fy_global = Fy_body; Fz_global = -Fz_body\n%%');
	fprintf(fid,'%% Mx_global = -Mx_body; My_global = My_body; Mz_global = -Mz_body\n%%');
	fprintf(fid,'%% --------------------------------------------------------------\n%%\n');
	fprintf(fid,'%% Alpha [rad]\n%f\n',alpha0);
	fprintf(fid,'%% Beta [rad]\n%f\n',beta0);

	fprintf(fid,'%%\n%% ------ Wing ------\n');
	fprintf(fid,'%% Lift [N]\n%f\n',Lwing);
	fprintf(fid,'%% Drag [N]\n%f\n',Dwing);
	fprintf(fid,'%% Side Force [N]\n%f\n',Swing);
	fprintf(fid,'%% X axis Force [N]\n%f\n',Fxwing);
	fprintf(fid,'%% Y axis Force [N]\n%f\n',Fywing);
	fprintf(fid,'%% Z axis Force [N]\n%f\n',Fzwing);
	fprintf(fid,'%% X axis Moment [Nm]\n%f\n',Mwing(1));
	fprintf(fid,'%% Y axis Moment [Nm]\n%f\n',Mwing(2));
	fprintf(fid,'%% Z axis Moment [Nm]\n%f\n',Mwing(3));

	fprintf(fid,'%%\n%% ------ Horizontal Tail ------\n');
	fprintf(fid,'%% Lift [N]\n%f\n',Lhtail);
	fprintf(fid,'%% Drag [N]\n%f\n',Dhtail);
	fprintf(fid,'%% Side Force [N]\n%f\n',Shtail);
	fprintf(fid,'%% X axis Force [N]\n%f\n',Fxhtail);
	fprintf(fid,'%% Y axis Force [N]\n%f\n',Fyhtail);
	fprintf(fid,'%% Z axis Force [N]\n%f\n',Fzhtail);
	fprintf(fid,'%% X axis Moment [Nm]\n%f\n',Mhtail(1));
	fprintf(fid,'%% Y axis Moment [Nm]\n%f\n',Mhtail(2));
	fprintf(fid,'%% Z axis Moment [Nm]\n%f\n',Mhtail(3));

	fprintf(fid,'%%\n%% ------ Vertical Tail ------\n');
	fprintf(fid,'%% Lift [N]\n%f\n',Lvtail);
	fprintf(fid,'%% Drag [N]\n%f\n',Dvtail);
	fprintf(fid,'%% Side Force [N]\n%f\n',Svtail);
	fprintf(fid,'%% X axis Force [N]\n%f\n',Fxvtail);
	fprintf(fid,'%% Y axis Force [N]\n%f\n',Fyvtail);
	fprintf(fid,'%% Z axis Force [N]\n%f\n',Fzvtail);
	fprintf(fid,'%% X axis Moment [Nm]\n%f\n',Mvtail(1));
	fprintf(fid,'%% Y axis Moment [Nm]\n%f\n',Mvtail(2));
	fprintf(fid,'%% Z axis Moment [Nm]\n%f\n',Mvtail(3));


	% Clearing some no-use vectors..
	clear Component_Lifts Component_Drags Component_SideForces Component_Moments
	clear Component_Fxs Component_Fys Component_Fzs
	clear Lwing Dwing Swing Mwing
	clear Lhtail Dhtail Shtail Mhtail
	clear Lvtail Dvtail Svtail Mvtail	

	fclose(fid);
end % if

% #####################################################################################################
% #####################################################################################################
% 			Panel Aerodynamic Force
% #####################################################################################################
% #####################################################################################################

if panel_aerodynamic_forces == 1
	fid = fopen('Calculated_Panel_Aerodynamic_Force.dat','w');

	fprintf(fid,'%% --------------------------------------------------------------\n');
	fprintf(fid,'%% This file contains the Aerodynamic Force on every panel, calculated\n');
	fprintf(fid,'%% in the conditions specified in the file "Initial_Conditions.m"\n');
	fprintf(fid,'%% x,y,z are the coordinates of the panel''s center, in global axis\n');
	fprintf(fid,'%% S is the Surface of the Panel\n');
	fprintf(fid,'%% Fx Fy Fz are the forces on the panel in BODY axis\n');
	fprintf(fid,'%% Cp is the coefficient of pressure\n');
	fprintf(fid,'%% ... of course you could obtain Cp from Fx,Fy,Fz and S.. I''m just helping you here :P \n');
	fprintf(fid,'%% if you need normals you can export the geometry using the relative flag in "What_To_Do.m"\n');
	fprintf(fid,'%% GLOBAL AXIS TO BODY AXIS.. and vice-versa:\n');
	fprintf(fid,'%% In general, forces and moments are calculated in BODY AXIS, which are in the same\n');
	fprintf(fid,'%% direction as global axis, BUT x_body is from CG to the nose, y_body is\n');
	fprintf(fid,'%% from CG to the right wing (exactly as y_global), z_body from CG to the \n');
	fprintf(fid,'%% bottom of the airplane.\n');
	fprintf(fid,'%% Moments are calculated in the refPoint position.\n');
	fprintf(fid,'%%\n%% Trasforming body axis into global axis, it''s pretty easy:\n');
	fprintf(fid,'%% Fx_global = -Fx_body; Fy_global = Fy_body; Fz_global = -Fz_body\n%%');
	fprintf(fid,'%% Mx_global = -Mx_body; My_global = My_body; Mz_global = -Mz_body\n%%');
	fprintf(fid,'%% --------------------------------------------------------------\n%%\n');
	fprintf(fid,'%% x [m] || y [m] || z [m] || S [m2] || Fx [N] || Fy [N] || Fz [N] || Cp [adim] \n');
	for jj = 1:numel(xcv)
		fprintf(fid,'%f %f %f %f %f %f %f %f\n',xcv(jj),ycv(jj),zcv(jj),surfvect(jj),dFVECT(jj,1),dFVECT(jj,2),dFVECT(jj,3),Cp(jj));
	end % for

	fclose(fid);
end % if


% Plot Force Vectors
if plot_cp_vectors == 1

	figure, hold on
	for jj = 1:numel(xshape)
		plot3(xshape{jj},yshape{jj},zshape{jj},'k','LineWidth',2)
		grid on
		axis equal
	end % for
	plot3(CG(1),CG(2),CG(3),'ok','LineWidth',5,'MarkerSize',7);
	quiver3(-2,0,0,cos(alpha0)*cos(beta0),cos(alpha0)*sin(beta0),sin(alpha0),'b','LineWidth',3)
	quiver3(xcv,ycv,zcv,Cp.*xnv,Cp.*ynv,Cp.*znv,'r')

	title('Pressure Coefficient Plot','FontSize',14)
	xlim([-2,6]), ylim([-(b+1),b+1]), zlim([-6,6])
	axis equal
	axis off
end


