% MODULE_Geometry_Exporting
% 
% This module exports the geometry created in terms of:
% -> position of panel's centers (control points)
% -> normal vectors
% -> panel surface area
% 
% What is to be exported depends on the flags activated in the file What_To_Do

fid = fopen('Geometry_Exported.dat','w');

fprintf(fid,'%% This is the geometry you exported..\n');
fprintf(fid,'%% every column represent the following: \n');
fprintf(fid,'%% (CP -> Control Point, V1 -> Vortex #1\n');

if control_points == 1, fprintf(fid,'%% x CP [m] || y CP [m] || z CP [m] ||'); end

if normal_vectors == 1, fprintf(fid,'%% nx || ny || nz ||'); end

if surface_area == 1, fprintf(fid,'%% S [m2] ||'); end

if vortex_corners == 1, fprintf(fid,'%% x V1 || y V1 || z V1 || x V2 || y V2 || z V2 || x V3 || y V3 || z V3 || x V4 || y V4 || z V4'); end

fprintf(fid,'\n\n');

for jj = 1:numel(xcv)
	
	if control_points == 1, fprintf(fid,'%f %f %f ',xcv(jj),ycv(jj),zcv(jj)); end

	if normal_vectors == 1,	fprintf(fid,'%f %f %f ',xnv(jj),ynv(jj),znv(jj)); end

	if surface_area == 1, fprintf(fid,'%f ',surfvect(jj)); end

	if vortex_corners == 1
		xv1 = xvmat(1,jj); yv1 = yvmat(1,jj); zv1 = zvmat(1,jj);
		xv2 = xvmat(2,jj); yv2 = yvmat(2,jj); zv2 = zvmat(2,jj);
		xv3 = xvmat(3,jj); yv3 = yvmat(3,jj); zv3 = zvmat(3,jj);
		xv4 = xvmat(4,jj); yv4 = yvmat(4,jj); zv4 = zvmat(4,jj);
		fprintf(fid,'%f %f %f %f %f %f %f %f %f %f %f %f ',xv1,yv1,zv1,xv2,yv2,zv2,xv3,yv3,zv3,xv4,yv4,zv4);
	end
	fprintf(fid,'\n');

end % for

fclose(fid);
