% MODULE Longitudinal Modes

global plott alpha0 beta0 Vinf rho

% Let's see if stability derivatives have been calculated yet
try
 CFy_beta;
catch outputtt;
 lateral_derivatives = 1;
 MODULE_Stability_Derivatives;
end

% we need mean aerodynamic chord
try 
 MAC_wing
catch outputtt;
 surfaces_and_aerodynamic_chords = 1;
 MODULE_Geometry_Various;
end


% Defining Coefficients
wingspan = 2*sum(bvect_wing);
mu 	= 2*m/(rho*Swing*MAC_wing);
CW0	= m*g/(0.5*1.225*Vinf^2*Swing);

Ix_hat 	= Ix/( rho*Swing*(wingspan/2)^3 );
Iz_hat 	= Iz/( rho*Swing*(wingspan/2)^3 );
Ixz_hat = Ixz/( rho*Swing*(wingspan/2)^3 );

Ix1_hat = wingspan/MAC_wing/Iz_hat * (Ix_hat*Iz_hat - Ixz_hat^2);
Iz1_hat = wingspan/MAC_wing/Ix_hat * (Ix_hat*Iz_hat - Ixz_hat^2);
Ixz1_hat = wingspan/MAC_wing/Ixz_hat * (Ix_hat*Iz_hat - Ixz_hat^2);

LAT_MAT1 = 1/2/mu* [CFy_beta, CFy_p, (CFy_r - 2*mu*MAC_wing/wingspan), CW0, 0];
LAT_MAT2 = 1/Ix1_hat*[CMx_beta,CMx_p,CMx_r,0,0] + 1/Ixz1_hat*[CMz_beta,CMz_p,CMz_r,0,0];
LAT_MAT3 = 1/Ixz1_hat*[CMx_beta,CMx_p,CMx_r,0,0] + 1/Iz1_hat*[CMz_beta,CMz_p,CMz_r,0,0];
LAT_MAT4 = [0,MAC_wing/wingspan,0,0,0];
LAT_MAT5 = [0,0,MAC_wing/wingspan,0,0];

LAT_MAT = [LAT_MAT1; LAT_MAT2; LAT_MAT3; LAT_MAT4; LAT_MAT5];

[EIGEN_lat,lambda_lat] = eig(LAT_MAT);
lambda_lat = diag(lambda_lat);

if eigenvalue_plot_lateral == 1
	
	lateral_eig_plot = figure;
	plot(real(lambda_lat), imag(lambda_lat), 'ok', 'LineWidth', 3, 'MarkerSize',5);	
	title('Eigenvalues for Lateral Plane Dynamics','FontSize',14);
	xlabel('Real()','FontSize',14), ylabel('Imag()','FontSize',14)
	axis equal
	grid on
end

if eigenvectors_plot_lateral == 1

	E11r = real(EIGEN_lat(1,1));
	E21r = real(EIGEN_lat(2,1));
	E31r = real(EIGEN_lat(3,1));
	E41r = real(EIGEN_lat(4,1));
	E51r = real(EIGEN_lat(5,1));

	E11i = imag(EIGEN_lat(1,1));
	E21i = imag(EIGEN_lat(2,1));
	E31i = imag(EIGEN_lat(3,1));
	E41i = imag(EIGEN_lat(4,1));
	E51i = imag(EIGEN_lat(5,1));


	E12r = real(EIGEN_lat(1,2));
	E22r = real(EIGEN_lat(2,2));
	E32r = real(EIGEN_lat(3,2));
	E42r = real(EIGEN_lat(4,2));
	E52r = real(EIGEN_lat(5,2));

	E12i = imag(EIGEN_lat(1,2));
	E22i = imag(EIGEN_lat(2,2));
	E32i = imag(EIGEN_lat(3,2));
	E42i = imag(EIGEN_lat(4,2));
	E52i = imag(EIGEN_lat(5,2));


	E13r = real(EIGEN_lat(1,3));
	E23r = real(EIGEN_lat(2,3));
	E33r = real(EIGEN_lat(3,3));
	E43r = real(EIGEN_lat(4,3));
	E53r = real(EIGEN_lat(5,3));

	E13i = imag(EIGEN_lat(1,3));
	E23i = imag(EIGEN_lat(2,3));
	E33i = imag(EIGEN_lat(3,3));
	E43i = imag(EIGEN_lat(4,3));
	E53i = imag(EIGEN_lat(5,3));


	E14r = real(EIGEN_lat(1,4));
	E24r = real(EIGEN_lat(2,4));
	E34r = real(EIGEN_lat(3,4));
	E44r = real(EIGEN_lat(4,4));
	E54r = real(EIGEN_lat(5,4));

	E14i = imag(EIGEN_lat(1,4));
	E24i = imag(EIGEN_lat(2,4));
	E34i = imag(EIGEN_lat(3,4));
	E44i = imag(EIGEN_lat(4,4));
	E54i = imag(EIGEN_lat(5,4));


	E15r = real(EIGEN_lat(1,5));
	E25r = real(EIGEN_lat(2,5));
	E35r = real(EIGEN_lat(3,5));
	E45r = real(EIGEN_lat(4,5));
	E55r = real(EIGEN_lat(5,5));

	E15i = imag(EIGEN_lat(1,5));
	E25i = imag(EIGEN_lat(2,5));
	E35i = imag(EIGEN_lat(3,5));
	E45i = imag(EIGEN_lat(4,5));
	E55i = imag(EIGEN_lat(5,5));

	lateral_eigenvectors_plot = figure;
	subplot(2,3,1)
	plot(E11r,E11i,'o',E21r,E21i,'o',E31r,E31i,'o',E41r,E41i,'o',E51r,E51i,'o','LineWidth',3);
	legend('beta','p','r','phi','psi');
	xlabel('Real()'),ylabel('Imag'), title('EigenVectors for lateral plane')
	grid on

	subplot(2,3,2)
	plot(E12r,E12i,'o',E22r,E22i,'o',E32r,E32i,'o',E42r,E42i,'o',E52r,E52i,'o','LineWidth',3);
	legend('beta','p','r','phi','psi');
	xlabel('Real()'),ylabel('Imag'), title('EigenVectors for lateral plane')
	grid on

	subplot(2,3,3)
	plot(E13r,E13i,'o',E23r,E23i,'o',E33r,E33i,'o',E43r,E43i,'o',E53r,E53i,'o','LineWidth',3);
	legend('beta','p','r','phi','psi');
	xlabel('Real()'),ylabel('Imag'), title('EigenVectors for lateral plane')
	grid on

	subplot(2,3,4)
	plot(E14r,E14i,'o',E24r,E24i,'o',E34r,E34i,'o',E44r,E44i,'o',E54r,E54i,'o','LineWidth',3);
	legend('beta','p','r','phi','psi');
	xlabel('Real()'),ylabel('Imag'), title('EigenVectors for lateral plane')
	grid on

	subplot(2,3,5)
	plot(E15r,E15i,'o',E25r,E25i,'o',E35r,E35i,'o',E45r,E45i,'o',E55r,E55i,'o','LineWidth',3);
	legend('beta','p','r','phi','psi');
	xlabel('Real()'),ylabel('Imag'), title('EigenVectors for lateral plane')
	grid on
	
end

if state_matrix_export_lateral == 1

	fid = fopen('Lateral_State_Matrix.dat','w');
	fprintf(fid,'%% This is the State Matrix of the lateral plane\n');
	NNN = size(LAT_MAT,1);
	for ii = 1:NNN
		for jj = 1:NNN
			fprintf(fid,'%f ',LAT_MAT(ii,jj));
		end
		fprintf(fid,'\n');
	end
			


	fclose(fid);
end 

