% MODULE Longitudinal Modes

global plott alpha0 beta0 Vinf rho

% Let's see if stability derivatives have been calculated yet
try
 CL_alpha;
catch outputtt;
 longitudinal_derivatives = 1;
 MODULE_Stability_Derivatives;
end



% Defining Coefficients
mu 	= 2*m/(rho*Swing*MAC_wing);
Iy_hat 	= Iy/( rho*Swing*(MAC_wing/2)^3 );
CW0	= m*g/(0.5*1.225*Vinf^2*Swing);
denn 	= 2*mu + CL_alphadot;

LONG_MAT1 = 0.5/mu*[-CD_V, (CL0-CD_alpha), 0, -CW0];
LONG_MAT2 = 1/denn*[-(CL_V+2*CW0),-(CL_alpha+CD0),(2*mu-CL_q),0];
LONG_MAT3 = 1/Iy_hat*[ CMy_V-CM_alphadot*(CL_V+2*CW0)/denn , CMy_alpha-CM_alphadot*(CD0+CL_alpha)/denn , CMy_q+CM_alphadot*(2*mu-CL_q)/denn , 0];
LONG_MAT4 = [0,0,1,0];

LONG_MAT = [LONG_MAT1;LONG_MAT2;LONG_MAT3;LONG_MAT4];

[EIGEN_long,lambda_long] = eig(LONG_MAT);
lambda_long = diag(lambda_long);

if eigenvalue_plot_longitudinal == 1
	
	longitudinal_eig_plot = figure;
	plot(real(lambda_long), imag(lambda_long), 'ok', 'LineWidth', 3, 'MarkerSize',5);	
	title('Eigenvalues for Longitudinal Plane Dynamics','FontSize',14);
	xlabel('Real()','FontSize',14), ylabel('Imag()','FontSize',14)
	axis equal
	grid on
end

if eigenvectors_plot_longitudinal == 1

	E11r = real(EIGEN_long(1,1));
	E21r = real(EIGEN_long(2,1));
	E31r = real(EIGEN_long(3,1));
	E41r = real(EIGEN_long(4,1));

	E11i = imag(EIGEN_long(1,1));
	E21i = imag(EIGEN_long(2,1));
	E31i = imag(EIGEN_long(3,1));
	E41i = imag(EIGEN_long(4,1));

	E13r = real(EIGEN_long(1,3));
	E23r = real(EIGEN_long(2,3));
	E33r = real(EIGEN_long(3,3));
	E43r = real(EIGEN_long(4,3));

	E13i = imag(EIGEN_long(1,3));
	E23i = imag(EIGEN_long(2,3));
	E33i = imag(EIGEN_long(3,3));
	E43i = imag(EIGEN_long(4,3));

	longitudinal_eigenvectors_plot = figure;
	subplot(1,2,1)
	plot(E11r,E11i,'o',E21r,E21i,'o',E31r,E31i,'o',E41r,E41i,'o','LineWidth',3);
	legend('V','alpha','q','theta');
	xlabel('Real()'),ylabel('Imag'), title('EigenVectors for longitudinal plane')
	grid on

	subplot(1,2,2)
	plot(E13r,E13i,'o',E23r,E23i,'o',E33r,E33i,'o',E43r,E43i,'o','LineWidth',3);
	legend('V','alpha','q','theta');
	xlabel('Real()'),ylabel('Imag'), title('EigenVectors for longitudinal plane')
	grid on
	
end

if state_matrix_export_longitudinal == 1
	fid = fopen('Longitudinal_State_Matrix.dat','w');
	fprintf(fid,'%% This is the State Matrix of the longitudinal plane\n');
	NNN = size(LONG_MAT,1);
	for ii = 1:NNN
		for jj = 1:NNN
			fprintf(fid,'%f ',LONG_MAT(ii,jj));
		end
		fprintf(fid,'\n');
	end
			


	fclose(fid);
end 

