% MODULE Stability Derivatives
%

global plott alpha0 beta0 Vinf rho

% Calculating Aerodynamic Coefficients Matrix, if not done yet!
try
	MAT;	% let's see if MAT already exists
catch outputtt;
	[MAT] = Vortex_Lattice_INDUCEDVELOCITYFUNCTION('');
end



% STEP 1) WE NEED AERODYNAMIC PROPERTIES!
% if not already calculated, we compute the Mean Aerodynamic Chord
try
	MAC_wing;
catch outputtt;
	surfaces_and_aerodynamic_chords		= 1;
	MODULE_Geometry_Various;
end

try
	xN_tail;
catch outputtt;
	aerodynamic_centers_lifting_derivatives	= 1;
	MODULE_Component_Aerodynamics;
end


Swing 	= sum(surfvectwing);
Shtail 	= sum(surfvecthtail);
Svtail	= sum(surfvectvtail);
VOL_htail = Shtail / Swing * (xN_htail - xN_wing) / MAC_wing;

a_htail = L_alpha_htail/(0.5*rho*Vinf^2*Shtail);



% #############################################################################################
% #############################################################################################
% 				LONGITUDINAL STABILITY DERIVATIVES
% #############################################################################################
% #############################################################################################


if longitudinal_derivatives == 1

	% Common refPoint for Moments calculation
	refPoint = CG;

	wingspan = 2*sum(bvect_wing);

	% ----------------------------- Linearization Condition --------------------------------------------
	[Fx0,Fy0,Fz0,M0,L0,D0] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,0,0,0,0,0,0,refPoint);

	CFx0 = Fx0/(0.5*rho*Vinf^2*Swing);
	CFy0 = Fy0/(0.5*rho*Vinf^2*Swing);
	CFz0 = Fz0/(0.5*rho*Vinf^2*Swing);

	CMx0 = M0(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CMy0 = M0(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CMz0 = M0(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CL0 = L0/(0.5*rho*Vinf^2*Swing);
	CD0 = D0/(0.5*rho*Vinf^2*Swing);


	% ------------------------ Alpha Derivatives ------------------------------------------------------------
	alpha1 = alpha0 - 1*pi/180;
	alpha2 = alpha0 + 1*pi/180;
	[Fx1,Fy1,Fz1,M1,L1,D1] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha1,beta0,0,0,0,0,0,0,refPoint);
	[Fx2,Fy2,Fz2,M2,L2,D2] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha2,beta0,0,0,0,0,0,0,refPoint);
	
	CFx1 = Fx1/(0.5*rho*Vinf^2*Swing);
	CFy1 = Fy1/(0.5*rho*Vinf^2*Swing);
	CFz1 = Fz1/(0.5*rho*Vinf^2*Swing);

	CFx2 = Fx2/(0.5*rho*Vinf^2*Swing);
	CFy2 = Fy2/(0.5*rho*Vinf^2*Swing);
	CFz2 = Fz2/(0.5*rho*Vinf^2*Swing);

	CMx1 = M1(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CMy1 = M1(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CMz1 = M1(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CMx2 = M2(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CMy2 = M2(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CMz2 = M2(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CL1 = L1/(0.5*rho*Vinf^2*Swing);
	CL2 = L2/(0.5*rho*Vinf^2*Swing);

	CD1 = D1/(0.5*rho*Vinf^2*Swing);
	CD2 = D2/(0.5*rho*Vinf^2*Swing);


	CFx_alpha = (CFx2 -CFx1)/(alpha2 - alpha1);
	CFy_alpha = (CFy2 -CFy1)/(alpha2 - alpha1);
	CFz_alpha = (CFz2 -CFz1)/(alpha2 - alpha1);
	
	CMx_alpha = (CMx2-CMx1)/(alpha2 - alpha1);
	CMy_alpha = (CMy2-CMy1)/(alpha2 - alpha1);
	CMz_alpha = (CMz2-CMz1)/(alpha2 - alpha1);

	CL_alpha = (CL2-CL1)/(alpha2-alpha1);
	CD_alpha = (CD2-CD1)/(alpha2-alpha1);


	% -----------------------------  V - wind speed Derivatives ----------------------------------------------
	uu1 = -Vinf/60 * cos(alpha0);	% [m/s]
	uu2 =  Vinf/60 * cos(alpha0);
	
	ww1 = -Vinf/60*sin(alpha0);
	ww2 =  Vinf/60*sin(alpha0);
	
	VV1 = -sqrt(uu1.^2 + ww1.^2);
	VV2 = sqrt(uu2.^2 + ww2.^2);
	
	[Fx1,Fy1,Fz1,M1,L1,D1] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,uu1,0,ww1,0,0,0,refPoint);
	[Fx2,Fy2,Fz2,M2,L2,D2] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,uu2,0,ww2,0,0,0,refPoint);
		

	CFx1 = Fx1/(0.5*rho*Vinf^2*Swing);
	CFy1 = Fy1/(0.5*rho*Vinf^2*Swing);
	CFz1 = Fz1/(0.5*rho*Vinf^2*Swing);

	CFx2 = Fx2/(0.5*rho*Vinf^2*Swing);
	CFy2 = Fy2/(0.5*rho*Vinf^2*Swing);
	CFz2 = Fz2/(0.5*rho*Vinf^2*Swing);

	CMx1 = M1(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CMy1 = M1(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CMz1 = M1(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CM2x = M2(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CM2y = M2(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CM2z = M2(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CL1 = L1/(0.5*rho*Vinf^2*Swing);
	CL2 = L2/(0.5*rho*Vinf^2*Swing);

	CD1 = D1/(0.5*rho*Vinf^2*Swing);
	CD2 = D2/(0.5*rho*Vinf^2*Swing);

	CFx_V = Vinf*(CFx2 -CFx1)/(VV2 - VV1);
	CFy_V = Vinf*(CFy2 -CFy1)/(VV2 - VV1);
	CFz_V = Vinf*(CFz2 -CFz1)/(VV2 - VV1);
	
	CMx_V = Vinf*(CMx2-CMx1)/(VV2 - VV1);
	CMy_V = Vinf*(CMy2-CMy1)/(VV2 - VV1);
	CMz_V = Vinf*(CMz2-CMz1)/(VV2 - VV1);

	CL_V = Vinf*(CL2-CL1)/(VV2-VV1);
	CD_V = Vinf*(CD2-CD1)/(VV2-VV1);

	
	% ----------------------------------- q - Pitch Rate Derivatives --------------------------------------------
	qq1 = -20*pi/180;	% rad/s
	qq2 = 20*pi/180;
	
	[Fx1,Fy1,Fz1,M1,L1,D1,S1] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,0,0,0,0,qq1,0,refPoint);
	[Fx2,Fy2,Fz2,M2,L2,D2,S2] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,0,0,0,0,qq2,0,refPoint);


	CFx1 = Fx1/(0.5*rho*Vinf^2*Swing);
	CFy1 = Fy1/(0.5*rho*Vinf^2*Swing);
	CFz1 = Fz1/(0.5*rho*Vinf^2*Swing);

	CFx2 = Fx2/(0.5*rho*Vinf^2*Swing);
	CFy2 = Fy2/(0.5*rho*Vinf^2*Swing);
	CFz2 = Fz2/(0.5*rho*Vinf^2*Swing);

	CMx1 = M1(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CMy1 = M1(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CMz1 = M1(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CM2x = M2(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CM2y = M2(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CM2z = M2(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CL1 = L1/(0.5*rho*Vinf^2*Swing);
	CL2 = L2/(0.5*rho*Vinf^2*Swing);

	CD1 = D1/(0.5*rho*Vinf^2*Swing);
	CD2 = D2/(0.5*rho*Vinf^2*Swing);

	CFx_q = 2*Vinf/MAC_wing*(CFx2 -CFx1)/(qq2 - qq1);
	CFy_q = 2*Vinf/MAC_wing*(CFy2 -CFy1)/(qq2 - qq1);
	CFz_q = 2*Vinf/MAC_wing*(CFz2 -CFz1)/(qq2 - qq1);
	
	CMx_q = 2*Vinf/MAC_wing*(CMx2-CMx1)/(qq2 - qq1);
	CMy_q = 2*Vinf/MAC_wing*(CMy2-CMy1)/(qq2 - qq1);
	CMz_q = 2*Vinf/MAC_wing*(CMz2-CMz1)/(qq2 - qq1);

	CL_q = 2*Vinf/MAC_wing*(CL2-CL1)/(qq2-qq1);
	CD_q = 2*Vinf/MAC_wing*(CD2-CD1)/(qq2-qq1);




	% ---------------------------------- alpha_dot Derivatives --------------------------------------------------

	CL_alphadot = 2*a_htail*VOL_htail * dEPSdALPHA;
	CM_alphadot = -2*a_htail*VOL_htail * dEPSdALPHA * (xN_htail-CG(1)) / MAC_wing;



	% =========================== WRITING ON A FILE ====================================
	fid = fopen('Stability_Derivatives_Longitudinal.dat','w');
	fprintf(fid,'%% This file contains the stability derivatives of the longitudinal plane.\n');
	fprintf(fid,'%% NOTE THAT ALPHA-DOT DERIVATIVES ARE NOT CALCULATED FOR BODY AXIS!!!!!\n');
	fprintf(fid,'%% .. i mean... EXCEPT FOR CM! All the values are expressed as NaN.\n');
	fprintf(fid,'%% So, don''t panic, NaN is my choice! :) \n');
	fprintf(fid,'%% ------------------------------------------------------------------------\n');
	fprintf(fid,'%%\n%% CFx_alpha || CFy_alpha || CFz_alpha || CMx_alpha || CMy_alpha || CMz_alpha || CL_alpha || CD_alpha\n');
	fprintf(fid,'%f %f %f %f %f %f %f %f\n',CFx_alpha,CFy_alpha,CFz_alpha,CMx_alpha,CMy_alpha,CMz_alpha,CL_alpha,CD_alpha);
	fprintf(fid,'%%\n%% CFx_V || CFy_V || CFz_V || CMx_V || CMy_V || CMz_V || CL_V || CD_V\n');
	fprintf(fid,'%f %f %f %f %f %f %f %f\n',CFx_V,CFy_V,CFz_V,CMx_V,CMy_V,CMz_V,CL_V,CD_V);
	fprintf(fid,'%%\n%% CFx_q || CFy_q || CFz_q || CMx_q || CMy_q || CMz_q || CL_q || CD_q\n');
	fprintf(fid,'%f %f %f %f %f %f %f %f\n',CFx_q,CFy_q,CFz_q,CMx_q,CMy_q,CMz_q,CL_q,CD_q);
	fprintf(fid,'%%\n%% CFx_adot || CFy_adot || CFz_adot || CMx_adot || CMy_adot || CMz_adot || CL_adot || CM_adot\n');
	fprintf(fid,'%f %f %f %f %f %f %f %f\n',NaN,NaN,NaN,NaN,CM_alphadot,NaN,CL_alphadot,NaN);
	fclose(fid);

end




% #############################################################################################
% #############################################################################################
% 				LATERAL STABILITY DERIVATIVES
% #############################################################################################
% #############################################################################################


if lateral_derivatives == 1

	
	%
	% ATTENTION: beta, p and r around global axis generate a rotation which is opposite to true body axis rotation
	% .. that's almost why we need a minus sign on the derivatives!
	%

	% Common refPoint for Moments calculation
	refPoint = CG;

	wingspan = 2*sum(bvect_wing);

	% ----------------------------- Linearization Condition --------------------------------------------
	[Fx0,Fy0,Fz0,M0,L0,D0] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,0,0,0,0,0,0,refPoint);

	CFx0 = Fx0/(0.5*rho*Vinf^2*Swing);
	CFy0 = Fy0/(0.5*rho*Vinf^2*Swing);
	CFz0 = Fz0/(0.5*rho*Vinf^2*Swing);

	CMx0 = M0(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CMy0 = M0(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CMz0 = M0(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CL0 = L0/(0.5*rho*Vinf^2*Swing);
	CD0 = D0/(0.5*rho*Vinf^2*Swing);


	% ------------------------ Beta Derivatives ------------------------------------------------------------
	beta1 = beta0 - 1*pi/180;
	beta2 = beta0 + 1*pi/180;
	[Fx1,Fy1,Fz1,M1,L1,D1] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta1,0,0,0,0,0,0,refPoint);
	[Fx2,Fy2,Fz2,M2,L2,D2] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta2,0,0,0,0,0,0,refPoint);
	
	CFx1 = Fx1/(0.5*rho*Vinf^2*Swing);
	CFy1 = Fy1/(0.5*rho*Vinf^2*Swing);
	CFz1 = Fz1/(0.5*rho*Vinf^2*Swing);

	CFx2 = Fx2/(0.5*rho*Vinf^2*Swing);
	CFy2 = Fy2/(0.5*rho*Vinf^2*Swing);
	CFz2 = Fz2/(0.5*rho*Vinf^2*Swing);

	CMx1 = M1(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CMy1 = M1(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CMz1 = M1(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CMx2 = M2(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CMy2 = M2(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CMz2 = M2(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CL1 = L1/(0.5*rho*Vinf^2*Swing);
	CL2 = L2/(0.5*rho*Vinf^2*Swing);

	CD1 = D1/(0.5*rho*Vinf^2*Swing);
	CD2 = D2/(0.5*rho*Vinf^2*Swing);


	CFx_beta = -(CFx2 -CFx1)/(beta2 - beta1);
	CFy_beta = -(CFy2 -CFy1)/(beta2 - beta1);
	CFz_beta = -(CFz2 -CFz1)/(beta2 - beta1);
	
	CMx_beta = -(CMx2-CMx1)/(beta2 - beta1);
	CMy_beta = -(CMy2-CMy1)/(beta2 - beta1);
	CMz_beta = -(CMz2-CMz1)/(beta2 - beta1);

	CL_beta = -(CL2-CL1)/(beta2-beta1);
	CD_beta = -(CD2-CD1)/(beta2-beta1);


	% ----------------------------------- p - Roll Rate Derivatives --------------------------------------------
	pp1 = -20*pi/180;	% rad/s
	pp2 = 20*pi/180;
	
	[Fx1,Fy1,Fz1,M1,L1,D1,S1] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,0,0,0,pp1,0,0,refPoint);
	[Fx2,Fy2,Fz2,M2,L2,D2,S2] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,0,0,0,pp2,0,0,refPoint);


	CFx1 = Fx1/(0.5*rho*Vinf^2*Swing);
	CFy1 = Fy1/(0.5*rho*Vinf^2*Swing);
	CFz1 = Fz1/(0.5*rho*Vinf^2*Swing);

	CFx2 = Fx2/(0.5*rho*Vinf^2*Swing);
	CFy2 = Fy2/(0.5*rho*Vinf^2*Swing);
	CFz2 = Fz2/(0.5*rho*Vinf^2*Swing);

	CMx1 = M1(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CMy1 = M1(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CMz1 = M1(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CM2x = M2(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CM2y = M2(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CM2z = M2(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CL1 = L1/(0.5*rho*Vinf^2*Swing);
	CL2 = L2/(0.5*rho*Vinf^2*Swing);

	CD1 = D1/(0.5*rho*Vinf^2*Swing);
	CD2 = D2/(0.5*rho*Vinf^2*Swing);

	CFx_p = -2*Vinf/wingspan*(CFx2 -CFx1)/(pp2 - pp1);
	CFy_p = -2*Vinf/wingspan*(CFy2 -CFy1)/(pp2 - pp1);
	CFz_p = -2*Vinf/wingspan*(CFz2 -CFz1)/(pp2 - pp1);
	
	CMx_p = -2*Vinf/wingspan*(CMx2-CMx1)/(pp2 - pp1);
	CMy_p = -2*Vinf/wingspan*(CMy2-CMy1)/(pp2 - pp1);
	CMz_p = -2*Vinf/wingspan*(CMz2-CMz1)/(pp2 - pp1);

	CL_p = -2*Vinf/wingspan*(CL2-CL1)/(pp2-pp1);
	CD_p = -2*Vinf/wingspan*(CD2-CD1)/(pp2-pp1);


	% ----------------------------------- r - Yaw Rate Derivatives --------------------------------------------
	rr1 = -20*pi/180;	% rad/s
	rr2 = 20*pi/180;
	
	[Fx1,Fy1,Fz1,M1,L1,D1,S1] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,0,0,0,0,0,rr1,refPoint);
	[Fx2,Fy2,Fz2,M2,L2,D2,S2] = Vortex_Lattice_SYSTEMSOLVERFUNCTION(MAT,alpha0,beta0,0,0,0,0,0,rr2,refPoint);


	CFx1 = Fx1/(0.5*rho*Vinf^2*Swing);
	CFy1 = Fy1/(0.5*rho*Vinf^2*Swing);
	CFz1 = Fz1/(0.5*rho*Vinf^2*Swing);

	CFx2 = Fx2/(0.5*rho*Vinf^2*Swing);
	CFy2 = Fy2/(0.5*rho*Vinf^2*Swing);
	CFz2 = Fz2/(0.5*rho*Vinf^2*Swing);

	CMx1 = M1(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CMy1 = M1(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CMz1 = M1(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CM2x = M2(1)/(0.5*rho*Vinf^2*Swing*wingspan);
	CM2y = M2(2)/(0.5*rho*Vinf^2*Swing*MAC_wing);
	CM2z = M2(3)/(0.5*rho*Vinf^2*Swing*wingspan);

	CL1 = L1/(0.5*rho*Vinf^2*Swing);
	CL2 = L2/(0.5*rho*Vinf^2*Swing);

	CD1 = D1/(0.5*rho*Vinf^2*Swing);
	CD2 = D2/(0.5*rho*Vinf^2*Swing);

	CFx_r = -2*Vinf/wingspan*(CFx2 -CFx1)/(rr2 - rr1);
	CFy_r = -2*Vinf/wingspan*(CFy2 -CFy1)/(rr2 - rr1);
	CFz_r = -2*Vinf/wingspan*(CFz2 -CFz1)/(rr2 - rr1);
	
	CMx_r = -2*Vinf/wingspan*(CMx2-CMx1)/(rr2 - rr1);
	CMy_r = -2*Vinf/wingspan*(CMy2-CMy1)/(rr2 - rr1);
	CMz_r = -2*Vinf/wingspan*(CMz2-CMz1)/(rr2 - rr1);

	CL_r = -2*Vinf/wingspan*(CL2-CL1)/(rr2-rr1);
	CD_r = -2*Vinf/wingspan*(CD2-CD1)/(rr2-rr1);



	% =========================== WRITING ON A FILE ====================================
	fid = fopen('Stability_Derivatives_Lateral.dat','w');
	fprintf(fid,'%% This file contains the stability derivatives of the lateral plane.\n');
	fprintf(fid,'%% Please, remember that CMx and CMz are adimensionalized with respect to\n');
	fprintf(fid,'%% the wing span, while CMy is with respect to the wing mean aerodynamic chord.\n');
	fprintf(fid,'%% ------------------------------------------------------------------------\n');
	fprintf(fid,'%%\n%% CFx_beta || CFy_beta || CFz_beta || CMx_beta || CMy_beta || CMz_beta || CL_beta || CD_beta\n');
	fprintf(fid,'%f %f %f %f %f %f %f %f\n',CFx_beta,CFy_beta,CFz_beta,CMx_beta,CMy_beta,CMz_beta,CL_beta,CD_beta);
	fprintf(fid,'%%\n%% CFx_p || CFy_p || CFz_p || CMx_p || CMy_p || CMz_p || CL_p || CD_p\n');
	fprintf(fid,'%f %f %f %f %f %f %f %f\n',CFx_p,CFy_p,CFz_p,CMx_p,CMy_p,CMz_p,CL_p,CD_p);
	fprintf(fid,'%%\n%% CFx_r || CFy_r || CFz_r || CMx_r || CMy_r || CMz_r || CL_r || CD_r\n');
	fprintf(fid,'%f %f %f %f %f %f %f %f\n',CFx_r,CFy_r,CFz_r,CMx_r,CMy_r,CMz_r,CL_r,CD_r);
	fclose(fid);

end

