% Vortex Lattice System Solver Function
%
% The function solves the linear system and computes forces and moments

function [Fx,Fy,Fz,Moment,Lift,Drag,SideForce,dFVECT,Cp] = Vortex_Lattice_SYSTEMSOLVERFUNCTION_HtailOnly(A,alpha,beta,uu,vv,ww,pp,qq,rr,refPoint);

global plott alpha0 beta0 Vinf rho

% Loading Geometry
plott = 0;

Geometry_Creator_HtailOnly

mod = sqrt( (xvmat(2,:)-xvmat(1,:)).^2 + (yvmat(2,:)-yvmat(1,:)).^2 +(zvmat(2,:)-zvmat(1,:)).^2 )';


% _________________________________________
% Creating Velocity Field
% _________________________________________
%
% To create velocity field, we define the components of the velocity vector in the position of
% every control points in the following way:
% first of all we add the asimptotic component,
% then for every rotational speed IN GLOBAL AXIS, we add the "velocity increment".
%
% alpha and beta are the linearization condition (input data!)

% Velocity in the linearization configuration
VinfX = Vinf*cos(alpha)*cos(beta);
VinfY = Vinf*cos(alpha)*sin(beta);
VinfZ = Vinf*sin(alpha);


% Adding delta u, delta v, delta w
VinfX = VinfX + uu;
VinfY = VinfY + vv;
VinfZ = VinfZ + ww;

% Adding rolling rate
VinfX = VinfX;
VinfY = VinfY + pp*(zcv-CG(3));
VinfZ = VinfZ - pp*(ycv-CG(2));

% Adding pitching rate
VinfX = VinfX - qq*(zcv-CG(3));
VinfY = VinfY;
VinfZ = VinfZ + qq*(xcv-CG(1));

% Adding yawing rate
VinfX = VinfX + rr*(ycv-CG(2));
VinfY = VinfY - rr*(xcv-CG(1));
VinfZ = VinfZ;

% Yes I could have done the previous steps in one only step.. but it's clearer this way i think :)




ff = VinfX'.*xnv + VinfY'.*ynv + VinfZ'.*znv;

% _______________________________________
% SOLVING THE LINEAR SISTEM
% _______________________________________


tic 
gamma = - A\ff;		% circulations
tt = toc;


%GammaWing = zeros(Nrightwing+Nleftwing+Nverttail+Nrighttail+Nlefttail,1);

GammaWing = zeros(N_htail,1);

% Horizontal Tail
for ii = 1:N_htail
	if numel( find(ii == [1:Nc_htail:N_htail]) ) == 0
		GammaWing(ii) = gamma(ii) - gamma(ii-1);
	else
		GammaWing(ii) = gamma(ii);
	end
end


% _____________________________________
% Rotation Matrices
% _____________________________________

% Rbw: Rotation Matrix from Body frame to Wind frame
% Rwb = Rbw^-1 = Rbw', 'cause R is orthogonal
Rbw1 = [cos(alpha)*cos(beta), -cos(alpha)*sin(beta), -sin(alpha)];
Rbw2 = [sin(beta), cos(beta), 0];
Rbw3 = [sin(alpha)*cos(beta), -sin(alpha)*sin(beta), cos(alpha)];
Rbw = [Rbw1; Rbw2; Rbw3];

n1w = Rbw*[1;0;0];
n2w = Rbw*[0;1;0];
n3w = Rbw*[0;0;1];

% _____________________________________
% FORCES ON EVERY PANEL
% _____________________________________

dF = rho*Vinf*GammaWing.*mod;
Cp = dF./surfvect;


dFVECT = [dF.*xnv,dF.*ynv,dF.*znv];	% made a matrix of it

dD = dFVECT * n1w;			% panel drag
dS = dFVECT * n2w;			% panel sideforce.. the minus sign is a mistery, but without it doesn't work properly..
dL = dFVECT * n3w;			% panel lift

arm = [xcv'- refPoint(1),ycv'- refPoint(2),zcv'-refPoint(3)];
dMVECT = cross(arm,dFVECT);	


% ATTENTION!!!!!!!!!!!!!!!!!!!!!!
% ATTENTION!!!!!!!!!!!!!!!!!!!!!!
% ATTENTION!!!!!!!!!!!!!!!!!!!!!!
% ATTENTION!!!!!!!!!!!!!!!!!!!!!!
% ATTENTION!!!!!!!!!!!!!!!!!!!!!!
% HERE I REDEFINE dFVECT IN BODY AXIS!!!!!
% I don't define dFVECT in body axis before, cause Lift and Drag are ok.. 
% Fx_body = -Fx_global; Fy_body = +Fy_global; Fz_body = -Fz_global;
% just... let it be
dFVECT = [-dF.*xnv,dF.*ynv,-dF.*znv];	% made a matrix of it
dMVECT = [-dMVECT(:,1),dMVECT(:,2),-dMVECT(:,3)];

% TOTAL FORCES
Lift = sum(dL);
Drag = sum(dD);
SideForce = sum(dS);
Moment = sum(dMVECT);

Fx = sum(dFVECT(:,1));
Fy = sum(dFVECT(:,2));
Fz = sum(dFVECT(:,3));


return
