% Vortex Lattice System Solver Function
%
% The function solves the linear system and computes forces and moments

function [Fx,Fy,Fz,Moment,Lift,Drag,SideForce,dFVECT,Cp,Epsilon] = Vortex_Lattice_SYSTEMSOLVERFUNCTION_WingOnly(A,alpha,beta,uu,vv,ww,pp,qq,rr,refPoint,POSvect);

global plott alpha0 beta0 Vinf rho

% Loading Geometry
plott = 0;

Geometry_Creator_WingOnly

mod = sqrt( (xvmat(2,:)-xvmat(1,:)).^2 + (yvmat(2,:)-yvmat(1,:)).^2 +(zvmat(2,:)-zvmat(1,:)).^2 )';


% _________________________________________
% Creating Velocity Field
% _________________________________________
%
% To create velocity field, we define the components of the velocity vector in the position of
% every control points in the following way:
% first of all we add the asimptotic component,
% then for every rotational speed IN GLOBAL AXIS, we add the "velocity increment".
%
% alpha and beta are the linearization condition (input data!)

% Velocity in the linearization configuration
VinfX = Vinf*cos(alpha)*cos(beta);
VinfY = Vinf*cos(alpha)*sin(beta);
VinfZ = Vinf*sin(alpha);


% Adding delta u, delta v, delta w
VinfX = VinfX + uu;
VinfY = VinfY + vv;
VinfZ = VinfZ + ww;

% Adding rolling rate
VinfX = VinfX;
VinfY = VinfY + pp*(zcv-CG(3));
VinfZ = VinfZ - pp*(ycv-CG(2));

% Adding pitching rate
VinfX = VinfX - qq*(zcv-CG(3));
VinfY = VinfY;
VinfZ = VinfZ + qq*(xcv-CG(1));

% Adding yawing rate
VinfX = VinfX + rr*(ycv-CG(2));
VinfY = VinfY - rr*(xcv-CG(1));
VinfZ = VinfZ;

% Yes I could have done the previous steps in one only step.. but it's clearer this way i think :)




ff = VinfX'.*xnv + VinfY'.*ynv + VinfZ'.*znv;

% _______________________________________
% SOLVING THE LINEAR SISTEM
% _______________________________________


tic 
gamma = - A\ff;		% circulations
tt = toc;


%GammaWing = zeros(Nrightwing+Nleftwing+Nverttail+Nrighttail+Nlefttail,1);
GammaWing = zeros(N_wing,1);

% Wing
for ii = 1:N_wing
	if numel( find(ii == [1:Nc_wing:N_wing]) ) == 0
		GammaWing(ii) = gamma(ii) - gamma(ii-1);
	else
		GammaWing(ii) = gamma(ii);
	end
end


% _____________________________________
% Rotation Matrices
% _____________________________________

% Rbw: Rotation Matrix from Body frame to Wind frame
% Rwb = Rbw^-1 = Rbw', 'cause R is orthogonal
Rbw1 = [cos(alpha)*cos(beta), -cos(alpha)*sin(beta), -sin(alpha)];
Rbw2 = [sin(beta), cos(beta), 0];
Rbw3 = [sin(alpha)*cos(beta), -sin(alpha)*sin(beta), cos(alpha)];
Rbw = [Rbw1; Rbw2; Rbw3];

n1w = Rbw*[1;0;0];
n2w = Rbw*[0;1;0];
n3w = Rbw*[0;0;1];

% _____________________________________
% FORCES ON EVERY PANEL
% _____________________________________

dF = rho*Vinf*GammaWing.*mod;
Cp = dF./surfvect;


dFVECT = [dF.*xnv,dF.*ynv,dF.*znv];	% made a matrix of it

dD = dFVECT * n1w;			% panel drag
dS = dFVECT * n2w;			% panel sideforce.. the minus sign is a mistery, but without it doesn't work properly..
dL = dFVECT * n3w;			% panel lift

arm = [xcv'- refPoint(1),ycv'- refPoint(2),zcv'-refPoint(3)];
dMVECT = cross(arm,dFVECT);	


% ATTENTION!!!!!!!!!!!!!!!!!!!!!!
% ATTENTION!!!!!!!!!!!!!!!!!!!!!!
% ATTENTION!!!!!!!!!!!!!!!!!!!!!!
% ATTENTION!!!!!!!!!!!!!!!!!!!!!!
% ATTENTION!!!!!!!!!!!!!!!!!!!!!!
% HERE I REDEFINE dFVECT IN BODY AXIS!!!!!
% I don't define dFVECT in body axis before, cause Lift and Drag are ok.. 
% Fx_body = -Fx_global; Fy_body = +Fy_global; Fz_body = -Fz_global;
% just... let it be
dFVECT = [-dF.*xnv,dF.*ynv,-dF.*znv];	% made a matrix of it
dMVECT = [-dMVECT(:,1),dMVECT(:,2),-dMVECT(:,3)];

% TOTAL FORCES
Lift = sum(dL);
Drag = sum(dD);
SideForce = sum(dS);
Moment = sum(dMVECT);

Fx = sum(dFVECT(:,1));
Fy = sum(dFVECT(:,2));
Fz = sum(dFVECT(:,3));

% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% ______________________________________________
% CALCULATING THE VELOCITY IN A POINT
% ______________________________________________
% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% ############################################################################################################
% This part of the code is intended to calculate the velocity induced in a point... This will be quite messy..
% ... be prepared
try
	POSvect;
	xP = POSvect(1); yP = POSvect(2); zP = POSvect(3);

		% _____________________________________
		% CREATING THE LINEAR SYSTEM
		% _____________________________________


		Ui = zeros(1,size(xvmat,2));
		Vi = zeros(1,size(xvmat,2));
		Wi = zeros(1,size(xvmat,2));

	
			for jj = 1:size(xvmat,2)

			r0_1 = [xvmat(2,jj)-xvmat(1,jj); yvmat(2,jj)-yvmat(1,jj);zvmat(2,jj)-zvmat(1,jj)];
			r1_1 = [xP-xvmat(1,jj); yP-yvmat(1,jj); zP-zvmat(1,jj)];
			r2_1 = [xP-xvmat(2,jj); yP-yvmat(2,jj); zP-zvmat(2,jj)];
		
			r0_2 = [xvmat(3,jj)-xvmat(2,jj); yvmat(3,jj)-yvmat(2,jj);zvmat(3,jj)-zvmat(2,jj)];
			r1_2 = [xP-xvmat(2,jj); yP-yvmat(2,jj); zP-zvmat(2,jj)];
			r2_2 = [xP-xvmat(3,jj); yP-yvmat(3,jj); zP-zvmat(3,jj)];

			r0_3 = [xvmat(4,jj)-xvmat(3,jj); yvmat(4,jj)-yvmat(3,jj);zvmat(4,jj)-zvmat(3,jj)];
			r1_3 = [xP-xvmat(3,jj); yP-yvmat(3,jj); zP-zvmat(3,jj)];
			r2_3 = [xP-xvmat(4,jj); yP-yvmat(4,jj); zP-zvmat(4,jj)];

			r0_4 = [xvmat(1,jj)-xvmat(4,jj); yvmat(1,jj)-yvmat(4,jj);zvmat(1,jj)-zvmat(4,jj)];
			r1_4 = [xP-xvmat(4,jj); yP-yvmat(4,jj); zP-zvmat(4,jj)];
			r2_4 = [xP-xvmat(1,jj); yP-yvmat(1,jj); zP-zvmat(1,jj)];

			r1xr2_1 = cross(r1_1,r2_1);
			r1xr2_2 = cross(r1_2,r2_2);
			r1xr2_3 = cross(r1_3,r2_3);
			r1xr2_4 = cross(r1_4,r2_4);

			VIND_1 = 1/(4*pi) * r1xr2_1 / norm(r1xr2_1)^2 * r0_1' * (r1_1/norm(r1_1) - r2_1/norm(r2_1));
			VIND_2 = 1/(4*pi) * r1xr2_2 / norm(r1xr2_2)^2 * r0_2' * (r1_2/norm(r1_2) - r2_2/norm(r2_2));
			VIND_3 = 1/(4*pi) * r1xr2_3 / norm(r1xr2_3)^2 * r0_3' * (r1_3/norm(r1_3) - r2_3/norm(r2_3));
			VIND_4 = 1/(4*pi) * r1xr2_4 / norm(r1xr2_4)^2 * r0_4' * (r1_4/norm(r1_4) - r2_4/norm(r2_4));
			VIND = VIND_1 + VIND_2 + VIND_3 + VIND_4;

			Ui(jj) = VIND(1)*GammaWing(jj);
			Vi(jj) = VIND(2)*GammaWing(jj);
			Wi(jj) = VIND(3)*GammaWing(jj);
	
		end

	U = sum(Ui); V = sum(Vi); W = sum(Wi);


	V_infinite = [ Vinf*cos(alpha)*cos(beta); Vinf*cos(alpha)*sin(beta); Vinf*sin(alpha)];
	V_deflected = [V_infinite(1)+U;V_infinite(2)+V;V_infinite(3)+W];

	Epsilon = acos(   V_infinite'*V_deflected / (norm(V_infinite)*norm(V_deflected))   );


catch outputtt;
	Epsilon = 6666666666666; % very very evil number.. to remember it's bullshit
end % try

return
