% 
% Syntax: [x_m,y_m] = naca4digit(m,p,s,N)
%
% The naca4digit function returns the mid-line coordinates of the NACA 4 digit airfoil whose 
% parameters are given as m,p,s,N
%
%
% p -> maximum camber position (%% chord)
% m -> maximum camber (%% chord)
% s -> maximum thickness


function [x_m,y_m] = naca4digit(m,p,s,N);

% step 1: defining a vector made of linspace() N+1 elements
x_m = 0:1/N:1;

% Chebishev nodes {uncomment to activate}:
%theta_par = pi:-pi/N:0;
%x_m = 0.5+cos(theta_par)/2;

% since values are inserted as percentual:
p = p/10; m = m/100; s = s/100;

% This is the expression of 4 digit NACA airfoil mid line
y_m2 = m/(1-p)^2*((1-2*p) + 2*p*x_m - x_m.^2+1e-10).*(x_m>=p);
y_m = y_m2;

if p > 0
	y_m1 = m/p^2*(2*p*x_m - x_m.^2).*(x_m<p);
	y_m = y_m + y_m1;
end


return
