function [nx,ny,nz,surfv,xc,yc,zc,xvortic,yvortic,zvortic,shape_mat,xyz_ldedge,xyz_tredge] = wing_geometry_function004(airfoil,rA,croot,ctip,b,lambda,delta,epsilon0,epsilon,Nc,Nb,FCR,xi,alpha00,beta00,SYM)

global plott alpha0 beta0 Vinf rho

% ##############################################################################
% ##############################################################################
% ##############################################################################
% RIGHT WING
% ##############################################################################
% ##############################################################################
% ##############################################################################

% Importing the profile
if numel(airfoil) ~= 8
 error('Wrong Airfoil Name! Only format ''NACAXXXX'' accepted'); 
end
m = str2num( airfoil(5) );
p = str2num( airfoil(6) );
s = str2num( airfoil(7:end) );

[xm,zm] = naca4digit(m,p,s,Nc);	% points on airfoil mid-line


% deflecting the flap..
if FCR > 1
	error('Flap Chord Ratio EXCEEDS UNITY!')
elseif FCR > 0
	RotationMAT = [cos(xi),sin(xi);-sin(xi),cos(xi)];

	% finding points To Be Rotated (tbr)
	pos_points_after_FCR = find( (xm-(1-FCR)) > 0);
	x_tbr = xm( pos_points_after_FCR );
	z_tbr = zm( pos_points_after_FCR );
	XZvv = RotationMAT*( [x_tbr;z_tbr] - repmat([x_tbr(1);z_tbr(1)],1,numel(x_tbr)) );	% rotating around the first point
	xm = [xm(1:pos_points_after_FCR-1) , XZvv(1,:)+x_tbr(1)];
	zm = [zm(1:pos_points_after_FCR-1) , XZvv(2,:)+z_tbr(1)];
end


% Pitch angle!
RotationMAT = [cos(epsilon0),sin(epsilon0);-sin(epsilon0),cos(epsilon0)];

XZvv = RotationMAT*[xm;zm];	% rotating around the first point
xm = XZvv(1,:);
zm = XZvv(2,:);




%if plott == 1, figure, plot(xm,zm), grid on, axis equal, end

% Converting input data rA to its components..
xA = rA(1);
yA = rA(2);
zA = rA(3);



% Leading edge

xB = xA + b*tan(lambda)+ctip/2*(1-cos(epsilon));
yB = yA + b;
zB = zA + b*tan(delta)+ctip/2*sin(epsilon);

% Trailing edge
xD = xA + croot; yD = yA; zD = zA;

xC = xA + b*tan(lambda)+ctip-ctip/2*(1-cos(epsilon));
yC = yA + b;
zC = zD + b*tan(delta)-ctip/2*sin(epsilon);


% _________________________________________
% Dividing the chord into panels

xroot = xA + xm*croot;
yroot = yA*ones(1,Nc+1);
zroot = zA + zm*croot;

xtip = xA + b*tan(lambda) + xm*croot*(xC-xB)/croot + ctip/2*(1-cos(epsilon));
ytip = yB*ones(1,Nc+1);
ztip = zB + zm*croot*ctip/croot - (xtip-xB)*sin(epsilon);


% Points at quarter chord
% x4cr -> x at 1/4 Chord on Root section
% x4ct -> x at 1/4 Chord on Tip section
x4cr = 1/4*xroot(2:end) + 3/4*xroot(1:end-1);
y4cr = 1/4*yroot(2:end) + 3/4*yroot(1:end-1);
z4cr = 1/4*zroot(2:end) + 3/4*zroot(1:end-1);

x4ct = 1/4*xtip(2:end) + 3/4*xtip(1:end-1);
y4ct = 1/4*ytip(2:end) + 3/4*ytip(1:end-1);
z4ct = 1/4*ztip(2:end) + 3/4*ztip(1:end-1);




% _________________________________________ 
% Creating Vortex Rings

% initializing point matrices
xmat = zeros(Nc+1,Nb+1); ymat = zeros(Nc+1,Nb+1); zmat = zeros(Nc+1,Nb+1);

for ii = 1:Nc
 [xmat(ii,:),ymat(ii,:),zmat(ii,:) ] = linspace3d([x4cr(ii),y4cr(ii),z4cr(ii)],[x4ct(ii),y4ct(ii),z4ct(ii)], Nb+1);
end
%xmat(end,:) = 300*croot;
%ymat(end,:) = ymat(end-1,:);
%zmat(end,:) = zmat(end-1,:);

xmat(end,:) = 300*cos(beta00)*croot + xmat(end-1,:);
ymat(end,:) = 300*sin(beta00)*croot + ymat(end-1,:);
zmat(end,:) = 300*sin(alpha00)*croot + zmat(end-1,:);

% _________________________________________ 
% Creating Normal Vectors and Control Points

% trailing edge vector, used to modify xmat, ymat, zmat
[xtredge,ytredge,ztredge] = linspace3d([xroot(end),yroot(end),zroot(end)],[xtip(end),ytip(end),ztip(end)],Nb+1);

xmod = [xmat(1:Nc,:); xtredge];
ymod = [ymat(1:Nc,:); ytredge];
zmod = [zmat(1:Nc,:); ztredge];


% making panel crossing vectors
xa = xmod(2:end, 2:end) - xmod(1:end-1,1:end-1);
ya = ymod(2:end, 2:end) - ymod(1:end-1,1:end-1);
za = zmod(2:end, 2:end) - zmod(1:end-1,1:end-1);

xb = xmod(1:end-1,2:end) - xmod(2:end,1:end-1);
yb = ymod(1:end-1,2:end) - ymod(2:end,1:end-1);
zb = zmod(1:end-1,2:end) - zmod(2:end,1:end-1);


% now some weird matricial operation...
aa = [xa(1:end)', ya(1:end)', za(1:end)'];
bb = [xb(1:end)', yb(1:end)', zb(1:end)'];


nn = cross(aa,bb);
norm_nn = sqrt(sum(nn.^2,2));
n = nn./repmat(norm_nn,1,3);

nx = n(:,1);
ny = n(:,2);
nz = n(:,3);



% _________________________________________ 
% Creating normal vectors to calculate the surface

% AFTER-TRAILING-EDGE vector, used to modify xmat, ymat, zmat
[xAFTtredge,yAFTtredge,zAFTtredge] = linspace3d([xroot(end)+1/4*croot/Nc,yroot(end),zroot(end)],[xtip(end)+1/4*ctip/Nc,ytip(end),ztip(end)],Nb+1);

xmod_surf = [xmat(1:Nc,:); xAFTtredge];
ymod_surf = [ymat(1:Nc,:); yAFTtredge];
zmod_surf = [zmat(1:Nc,:); zAFTtredge];


% making panel crossing vectors
xa = xmod_surf(2:end, 2:end) - xmod_surf(1:end-1,1:end-1);
ya = ymod_surf(2:end, 2:end) - ymod_surf(1:end-1,1:end-1);
za = zmod_surf(2:end, 2:end) - zmod_surf(1:end-1,1:end-1);

xb = xmod_surf(1:end-1,2:end) - xmod_surf(2:end,1:end-1);
yb = ymod_surf(1:end-1,2:end) - ymod_surf(2:end,1:end-1);
zb = zmod_surf(1:end-1,2:end) - zmod_surf(2:end,1:end-1);


% now some weird matricial operation...
aa = [xa(1:end)', ya(1:end)', za(1:end)'];
bb = [xb(1:end)', yb(1:end)', zb(1:end)'];


nn_surf = cross(aa,bb);
norm_nn_surf = sqrt(sum(nn_surf.^2,2));

% Surface
surfv = norm_nn_surf/2;	% "/2" because the vector product of the diagonals is twice the area of the quadrilateral!

% Control Points
xc = xmat(1:end-1,1:end-1) + xa/2;
yc = ymat(1:end-1,1:end-1) + ya/2;
zc = zmat(1:end-1,1:end-1) + za/2;


% LEADING EDGE AND TRAILING EDGE
[xldedge,yldedge,zldedge] = linspace3d([xroot(1),yroot(1),zroot(1)],[xtip(1),ytip(1),ztip(1)],Nb+1);
[xtredge,ytredge,ztredge] = linspace3d([xroot(end),yroot(end),zroot(end)],[xtip(end),ytip(end),ztip(end)],Nb+1);

xyz_ldedge = [xldedge;yldedge;zldedge];
xyz_tredge = [xtredge;ytredge;ztredge];


% RESUMING MATRICES
xAA = xmat(1:end-1,1:end-1); xAA = xAA(1:end);
yAA = ymat(1:end-1,1:end-1); yAA = yAA(1:end);
zAA = zmat(1:end-1,1:end-1); zAA = zAA(1:end);

xBB = xmat(1:end-1,2:end); xBB = xBB(1:end);
yBB = ymat(1:end-1,2:end); yBB = yBB(1:end);
zBB = zmat(1:end-1,2:end); zBB = zBB(1:end);

xCC = xmat(2:end,2:end); xCC = xCC(1:end);
yCC = ymat(2:end,2:end); yCC = yCC(1:end);
zCC = zmat(2:end,2:end); zCC = zCC(1:end);

xDD = xmat(2:end,1:end-1); xDD = xDD(1:end);
yDD = ymat(2:end,1:end-1); yDD = yDD(1:end);
zDD = zmat(2:end,1:end-1); zDD = zDD(1:end);

xvortic = [xAA;xBB;xCC;xDD];
yvortic = [yAA;yBB;yCC;yDD];
zvortic = [zAA;zBB;zCC;zDD];

if FCR == 0
	
	xshape = [xA,xtip,xroot(end:-1:1),xA];
	yshape = [yA,ytip,yroot(end:-1:1),yA];
	zshape = [zA,ztip,zroot(end:-1:1),zA];

elseif FCR > 0		% if a flap is present, plot it!

	PPAF = pos_points_after_FCR;
	xshape = [xA,xtip,xroot(PPAF(end:-1:1)),xtip(PPAF(1)),xroot(PPAF(1)),xroot(PPAF(1):-1:1)];
	yshape = [yA,ytip,yroot(PPAF(end:-1:1)),ytip(PPAF(1)),yroot(PPAF(1)),yroot(PPAF(1):-1:1)];
	zshape = [zA,ztip,zroot(PPAF(end:-1:1)),ztip(PPAF(1)),zroot(PPAF(1)),zroot(PPAF(1):-1:1)];

end

shape_mat = [xshape;yshape;zshape];

return
